"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const openai_1 = __importDefault(require("openai"));
const index_1 = require("../src/index");
const config_1 = require("./config");
const zod_1 = require("zod");
async function run() {
    const prompt = process.argv[2];
    const openai = new openai_1.default({ apiKey: config_1.OPENAI_API_KEY });
    const tools = [
        {
            type: 'function',
            function: {
                name: 'answer',
                description: 'Answer the question as precisely as possible',
                parameters: zod_1.z.object({
                    response: zod_1.z.string().min(1).describe('The response to show to the user'),
                }),
            },
        },
    ];
    const chat = new index_1.StructuredChat({
        openai,
        params: {
            model: 'gpt-4-turbo-preview',
            seed: config_1.OPENAI_SEED,
            tools,
        },
    });
    const response = await chat.send({
        messages: [{ role: 'user', content: prompt }],
        tool_choice: 'function',
    });
    const func = response.choices[0].message.tool_calls?.[0].function;
    console.log(func?.arguments);
}
run().catch((err) => {
    console.error(err);
    process.exit(1);
});
//# sourceMappingURL=index.js.map