"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileSystemCacheAdapter = void 0;
const path_1 = __importDefault(require("path"));
const promises_1 = __importDefault(require("fs/promises"));
class FileSystemCacheAdapter {
    options;
    constructor(options) {
        this.options = options;
    }
    async get(hash) {
        const cachePath = this._cachePath(hash);
        if (!(await this._hasCache(cachePath))) {
            return null;
        }
        return JSON.parse(await this._getCache(cachePath));
    }
    async set(hash, value) {
        const cachePath = this._cachePath(hash);
        await this._setCache(cachePath, JSON.stringify(value));
    }
    _cachePath(hash) {
        return path_1.default.join(this.options.baseDirectory, `${hash}.json`);
    }
    async _hasCache(cachePath) {
        try {
            await promises_1.default.access(cachePath, promises_1.default.constants.F_OK);
            return true;
        }
        catch {
            return false;
        }
    }
    async _getCache(cachePath) {
        return await promises_1.default.readFile(cachePath, 'utf-8');
    }
    async _setCache(cachePath, value) {
        return await promises_1.default.writeFile(cachePath, value, 'utf-8');
    }
}
exports.FileSystemCacheAdapter = FileSystemCacheAdapter;
//# sourceMappingURL=fileSystem.js.map