import type OpenAI from 'openai';
import type { z } from 'zod';
import { StructuredChatErrorCode } from './errors';
import { ChatCompletion, ChatCompletionAssistantMessageParam, ChatCompletionCreateParams, ChatCompletionMessageParam, ChatCompletionTool, ChatCompletionToolChoiceOption, RequestOptions, WithOptional, WithRequired, CacheAdapter, OpenAIChatCompletion } from './types';
export * from './cacheAdapters/fileSystem';
export * from './errors';
export * from './types';
export interface StructuredChatOptions<TTools extends Array<ChatCompletionTool>> {
    openai: OpenAI;
    cacheAdapter?: CacheAdapter;
    params: WithOptional<ChatCompletionCreateParams<TTools>, 'messages'>;
    options?: Omit<RequestOptions, '_attempt'>;
}
export declare class StructuredChat<TTools extends Array<ChatCompletionTool>> {
    openai: OpenAI;
    cacheAdapter?: CacheAdapter;
    params: WithOptional<ChatCompletionCreateParams<TTools>, 'messages'>;
    options?: Omit<RequestOptions, '_attempt'>;
    _messages: ChatCompletionCreateParams<TTools>['messages'];
    constructor(options: StructuredChatOptions<TTools>);
    send(params: WithOptional<Omit<ChatCompletionCreateParams<TTools>, 'tools'>, 'model'>, options?: RequestOptions): Promise<ChatCompletion<TTools>>;
    _send(params: WithOptional<Omit<ChatCompletionCreateParams<TTools>, 'tools'>, 'model'>, options?: RequestOptions): Promise<ChatCompletion<TTools>>;
    _sendErrorCorrection(result: OpenAI.ChatCompletion, responseMessages: Array<ChatCompletionMessageParam<TTools>>, params: WithOptional<Omit<ChatCompletionCreateParams<TTools>, 'tools'>, 'model'>, options: WithRequired<RequestOptions, '_attempt'>): Promise<{
        usage: {
            completion_tokens: number;
            prompt_tokens: number;
            total_tokens: number;
            requests: number;
        };
        object: "chat.completion";
        model: string;
        id: string;
        created: number;
        system_fingerprint?: string | undefined;
        choices: (Omit<OpenAI.Chat.Completions.ChatCompletion.Choice, "message"> & {
            message: ChatCompletionAssistantMessageParam<TTools>;
        })[];
    }>;
    _create(body: OpenAI.ChatCompletionCreateParamsNonStreaming, options?: OpenAI.RequestOptions): Promise<OpenAIChatCompletion>;
    _md5(value: unknown): Promise<string>;
    setMessages(messages: ChatCompletionCreateParams<TTools>['messages']): void;
    addMessage(message: ChatCompletionCreateParams<TTools>['messages'][number]): void;
    getMessages(): ChatCompletionMessageParam<TTools>[];
    _encodeMessage(message: ChatCompletionMessageParam<Array<ChatCompletionTool>> | OpenAI.ChatCompletionMessageParam): OpenAI.ChatCompletionMessageParam;
    _encodeTool(tool: ChatCompletionTool): OpenAI.ChatCompletionTool;
    _zodToJsonSchema(schema: z.ZodSchema): import("zod-to-json-schema").JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: import("zod-to-json-schema").JsonSchema7Type;
        } | undefined;
    };
    _deepRemoveKeys<T>(schema: T, exclude: Array<string>): T;
    _encodeToolChoice(tools: Array<ChatCompletionTool>, toolChoice: ChatCompletionToolChoiceOption<TTools>): OpenAI.ChatCompletionToolChoiceOption;
    _parseToolCall(tools: Array<ChatCompletionTool>, toolChoice: OpenAI.ChatCompletionToolChoiceOption, toolCall: OpenAI.ChatCompletionMessageToolCall): {
        success: false;
        errorCode: StructuredChatErrorCode;
        errorMessage: string;
    } | {
        success: true;
        arguments: unknown;
    };
    _parseGptJson(value: string): {
        success: true;
        data: any;
    } | {
        success: false;
        data?: undefined;
    };
}
