"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StructuredChat = void 0;
const flocky_1 = require("@devoxa/flocky");
const crypto_1 = __importDefault(require("crypto"));
const zod_to_json_schema_1 = require("zod-to-json-schema");
const errors_1 = require("./errors");
__exportStar(require("./cacheAdapters/fileSystem"), exports);
__exportStar(require("./errors"), exports);
__exportStar(require("./types"), exports);
class StructuredChat {
    openai;
    cacheAdapter;
    params;
    options;
    _messages = [];
    constructor(options) {
        this.openai = options.openai;
        this.cacheAdapter = options.cacheAdapter;
        this.params = options.params;
        this.options = options.options;
    }
    async send(params, options) {
        const result = await this._send(params, options);
        this._messages.push(...params.messages);
        this._messages.push(result.choices[0].message);
        return result;
    }
    async _send(params, options) {
        const _params = { ...this.params, ...params };
        const _options = { maxErrorCorrectionTries: 2, ...this.options, _attempt: 0, ...options };
        const tools = _params.tools || [];
        const messages = this._messages.concat(_params.messages);
        const encodedMessages = messages.map((message) => this._encodeMessage(message));
        const encodedTools = _params.tools
            ? _params.tools.map((tool) => this._encodeTool(tool))
            : undefined;
        const encodedToolChoice = _params.tool_choice
            ? this._encodeToolChoice(tools, _params.tool_choice)
            : undefined;
        const result = await this._create({
            ..._params,
            messages: encodedMessages,
            tools: encodedTools,
            tool_choice: encodedToolChoice,
        }, (0, flocky_1.omit)(_options, ['maxErrorCorrectionTries', '_attempt']));
        result.choices = result.choices.slice(0, 1);
        const toolCalls = result.choices[0].message.tool_calls;
        const hasFunctionCall = toolCalls && toolCalls.length > 0;
        const hasToHaveFunctionCall = _params.tool_choice === 'function' || typeof encodedToolChoice === 'object';
        if (!hasFunctionCall && hasToHaveFunctionCall) {
            const name = typeof encodedToolChoice === 'object' ? encodedToolChoice.function.name : null;
            const _errorMessage = name
                ? `You did not call a function. Respond by calling the "${name}" function.`
                : `You did not call a function. Respond by calling an available function.`;
            if (_options._attempt < _options.maxErrorCorrectionTries) {
                const responseMessages = [{ role: 'user', content: _errorMessage }];
                return this._sendErrorCorrection(result, responseMessages, _params, _options);
            }
            else {
                throw new errors_1.StructuredChatError('FUNCTION_CALL_NO_FUNCTION_CALLED', _errorMessage);
            }
        }
        if (hasFunctionCall) {
            const parsedToolCalls = toolCalls.map((toolCall) => this._parseToolCall(tools, encodedToolChoice || 'auto', toolCall));
            const parseFailure = parsedToolCalls.find((x) => !x.success);
            if (parseFailure && !parseFailure.success) {
                if (_options._attempt < _options.maxErrorCorrectionTries) {
                    const responseMessages = parsedToolCalls.map((toolCall, index) => ({
                        role: 'tool',
                        tool_call_id: toolCalls[index].id,
                        content: !toolCall.success ? toolCall.errorMessage : 'OK',
                    }));
                    return this._sendErrorCorrection(result, responseMessages, _params, _options);
                }
                else {
                    throw new errors_1.StructuredChatError(parseFailure.errorCode, parseFailure.errorMessage);
                }
            }
            for (let i = 0; i !== toolCalls.length; i++) {
                const parsedToolCall = parsedToolCalls[i];
                if (!parsedToolCall.success)
                    continue;
                toolCalls[i] = {
                    ...toolCalls[i],
                    function: {
                        ...toolCalls[i].function,
                        arguments: parsedToolCall.arguments,
                    },
                };
            }
        }
        const parsedResult = result;
        return { ...parsedResult, usage: { ...parsedResult.usage, requests: 1 } };
    }
    async _sendErrorCorrection(result, responseMessages, params, options) {
        const messages = [
            ...params.messages,
            result.choices[0].message,
            ...responseMessages,
        ];
        const correctResult = await this._send({ ...params, messages }, { ...options, _attempt: options._attempt + 1 });
        return {
            ...correctResult,
            usage: {
                completion_tokens: correctResult.usage.completion_tokens + result.usage.completion_tokens,
                prompt_tokens: correctResult.usage.prompt_tokens + result.usage.prompt_tokens,
                total_tokens: correctResult.usage.total_tokens + result.usage.total_tokens,
                requests: correctResult.usage.requests + 1,
            },
        };
    }
    async _create(body, options) {
        const hash = await this._md5({ body, options });
        if (this.cacheAdapter) {
            const value = await this.cacheAdapter.get(hash);
            if (value)
                return value;
        }
        const result = await this.openai.chat.completions.create(body, options);
        if (this.cacheAdapter) {
            await this.cacheAdapter.set(hash, JSON.parse(JSON.stringify(result)));
        }
        return result;
    }
    async _md5(value) {
        return crypto_1.default.createHash('md5').update(JSON.stringify(value)).digest('hex');
    }
    setMessages(messages) {
        this._messages = messages;
    }
    addMessage(message) {
        this._messages.push(message);
    }
    getMessages() {
        return this._messages;
    }
    _encodeMessage(message) {
        if (message.role !== 'assistant') {
            return message;
        }
        return {
            ...message,
            tool_calls: message.tool_calls
                ? message.tool_calls.map((toolCall) => ({
                    ...toolCall,
                    function: {
                        ...toolCall.function,
                        arguments: typeof toolCall.function.arguments === 'string'
                            ? toolCall.function.arguments
                            : JSON.stringify(toolCall.function.arguments),
                    },
                }))
                : undefined,
        };
    }
    _encodeTool(tool) {
        return {
            ...tool,
            function: {
                ...tool.function,
                parameters: tool.function.parameters
                    ? this._zodToJsonSchema(tool.function.parameters)
                    : undefined,
            },
        };
    }
    _zodToJsonSchema(schema) {
        const jsonSchema = (0, zod_to_json_schema_1.zodToJsonSchema)(schema);
        return this._deepRemoveKeys(jsonSchema, ['$schema', 'additionalProperties']);
    }
    _deepRemoveKeys(schema, exclude) {
        if (schema instanceof Array) {
            return schema.map((value) => this._deepRemoveKeys(value, exclude));
        }
        if (typeof schema === 'object') {
            const entries = Object.entries(schema)
                .filter(([key]) => !exclude.includes(key))
                .map(([key, value]) => [key, this._deepRemoveKeys(value, exclude)]);
            return Object.fromEntries(entries);
        }
        return schema;
    }
    _encodeToolChoice(tools, toolChoice) {
        if (toolChoice === 'function') {
            return tools.length > 1
                ? 'auto'
                : { type: 'function', function: { name: tools[0].function.name } };
        }
        return toolChoice;
    }
    _parseToolCall(tools, toolChoice, toolCall) {
        const name = toolCall.function.name;
        if (typeof toolChoice === 'object' && toolChoice.function.name !== name) {
            return {
                success: false,
                errorCode: 'FUNCTION_CALL_INCORRECT_FUNCTION_CALLED',
                errorMessage: `You did not call the correct function. Call the "${toolChoice.function.name}" function instead.`,
            };
        }
        const tool = tools.find((x) => x.function.name === name);
        if (!tool) {
            return {
                success: false,
                errorCode: 'FUNCTION_CALL_NONEXISTENT_FUNCTION_CALLED',
                errorMessage: `The function "${name}" does not exist. Only use the available functions.`,
            };
        }
        const schema = tool.function.parameters;
        if (!schema) {
            return { success: true, arguments: undefined };
        }
        const args = this._parseGptJson(toolCall.function.arguments);
        if (!args.success) {
            return {
                success: false,
                errorCode: 'FUNCTION_ARGUMENTS_MALFORMED_JSON',
                errorMessage: `The function "${name}" was called with a malformed JSON value as arguments. ` +
                    `Correct the JSON value to make it parsable and call the "${name}" function again.`,
            };
        }
        const parsedArgs = schema.safeParse(args.data);
        if (!parsedArgs.success) {
            const errors = parsedArgs.error.errors.map((error) => `- \`.${error.path.join('.')}\`: ${error.message}`);
            return {
                success: false,
                errorCode: 'FUNCTION_ARGUMENTS_INVALID_SCHEMA',
                errorMessage: `The function "${name}" was called with invalid arguments. ` +
                    `Apply these fixes and call the "${name}" function again:\n` +
                    errors.join('\n'),
            };
        }
        return {
            success: true,
            arguments: parsedArgs.data,
        };
    }
    _parseGptJson(value) {
        try {
            return { success: true, data: JSON.parse(value) };
        }
        catch (_) {
            return { success: false };
        }
    }
}
exports.StructuredChat = StructuredChat;
//# sourceMappingURL=index.js.map