import type OpenAI from 'openai';
import type { z } from 'zod';
export interface ChatCompletionCreateParams<TTools extends Array<ChatCompletionTool>> extends Omit<OpenAI.Chat.Completions.ChatCompletionCreateParamsNonStreaming, 'model' | 'messages' | 'tools' | 'tool_choice' | 'functions' | 'function_call'> {
    model: 'gpt-3.5-turbo' | 'gpt-4-turbo-preview';
    messages: Array<ChatCompletionMessageParam<TTools>>;
    tools?: TTools;
    tool_choice?: ChatCompletionToolChoiceOption<TTools>;
}
export type ChatCompletionMessageParam<TTools extends Array<ChatCompletionTool>> = OpenAI.ChatCompletionSystemMessageParam | OpenAI.ChatCompletionUserMessageParam | ChatCompletionAssistantMessageParam<TTools> | OpenAI.ChatCompletionToolMessageParam | OpenAI.ChatCompletionFunctionMessageParam;
export interface ChatCompletionAssistantMessageParam<TTools extends Array<ChatCompletionTool>> {
    role: 'assistant';
    content?: string | null;
    name?: string;
    tool_calls?: Array<ChatCompletionMessageToolCall<TTools>>;
}
interface ChatCompletionMessageToolCall<TTools extends Array<ChatCompletionTool>> {
    id: string;
    type: 'function';
    function: ChatCompletionMessageToolCallFunction<TTools>;
}
type ChatCompletionMessageToolCallFunction<TTools extends Array<ChatCompletionTool>> = ChatCompletionMessageToolCallFunctionGenerator<TTools[number]>[TTools[number]['function']['name']];
type ChatCompletionMessageToolCallFunctionGenerator<T extends {
    function: {
        name: string;
        parameters: z.ZodSchema | null;
    };
}> = {
    [K in T as K['function']['name']]: {
        name: K['function']['name'];
        arguments: K['function']['parameters'] extends null ? undefined : z.infer<NonNullable<K['function']['parameters']>>;
    };
};
export interface ChatCompletionTool {
    type: 'function';
    function: ChatCompletionToolFunctionDefinition;
}
interface ChatCompletionToolFunctionDefinition {
    name: string;
    description: string;
    parameters: z.ZodSchema | null;
}
export type ChatCompletionToolChoiceOption<TTools extends Array<ChatCompletionTool>> = 'none' | 'auto' | 'function' | ChatCompletionNamedToolChoice<TTools>;
interface ChatCompletionNamedToolChoice<TTools extends Array<ChatCompletionTool>> {
    type: 'function';
    function: ChatCompletionNamedToolChoiceFunction<TTools>;
}
interface ChatCompletionNamedToolChoiceFunction<TTools extends Array<ChatCompletionTool>> {
    name: TTools[number]['function']['name'];
}
export interface RequestOptions extends OpenAI.RequestOptions {
    timeout?: number;
    maxRetries?: number;
    maxErrorCorrectionTries?: number;
    _attempt?: number;
}
export type ChatCompletion<TTools extends Array<ChatCompletionTool>> = Omit<OpenAI.ChatCompletion, 'choices' | 'usage'> & {
    choices: Array<Omit<OpenAI.ChatCompletion.Choice, 'message'> & {
        message: ChatCompletionAssistantMessageParam<TTools>;
    }>;
    usage: OpenAI.ChatCompletion['usage'] & {
        requests: number;
    };
};
export type OpenAIChatCompletion = OpenAI.ChatCompletion;
export declare abstract class CacheAdapter {
    get: (hash: string) => Promise<OpenAIChatCompletion | null>;
    set: (hash: string, value: OpenAIChatCompletion) => Promise<void>;
}
export type WithOptional<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
export type WithRequired<T, K extends keyof T> = T & {
    [P in K]-?: T[P];
};
export {};
