import OpenAI from 'openai'
import { StructuredChat } from '../src/index'
import { OPENAI_API_KEY, OPENAI_SEED } from './config'
import { z } from 'zod'

async function run() {
  const prompt = process.argv[2]

  const openai = new OpenAI({ apiKey: OPENAI_API_KEY })

  const tools = [
    {
      type: 'function' as const,
      function: {
        name: 'answer' as const,
        description: 'Answer the question as precisely as possible',
        parameters: z.object({
          /** The response to show to the user */
          response: z.string().min(1).describe('The response to show to the user'),
        }),
      },
    },
  ]

  const chat = new StructuredChat({
    openai,
    params: {
      model: 'gpt-4-turbo-preview',
      seed: OPENAI_SEED,
      tools,
    },
  })

  const response = await chat.send({
    messages: [{ role: 'user', content: prompt }],
    tool_choice: 'function',
  })

  const func = response.choices[0].message.tool_calls?.[0].function
  console.log(func?.arguments)
}

run().catch((err) => {
  console.error(err)
  process.exit(1)
})
