import { CacheAdapter, OpenAIChatCompletion } from '../types'
import path from 'path'
import fs from 'fs/promises'

export interface FileSystemCacheAdapterOptions {
  baseDirectory: string
}

export class FileSystemCacheAdapter implements CacheAdapter {
  options: FileSystemCacheAdapterOptions

  constructor(options: FileSystemCacheAdapterOptions) {
    this.options = options
  }

  async get(hash: string): Promise<OpenAIChatCompletion | null> {
    const cachePath = this._cachePath(hash)

    if (!(await this._hasCache(cachePath))) {
      return null
    }

    return JSON.parse(await this._getCache(cachePath))
  }

  async set(hash: string, value: OpenAIChatCompletion): Promise<void> {
    const cachePath = this._cachePath(hash)
    await this._setCache(cachePath, JSON.stringify(value))
  }

  _cachePath(hash: string) {
    return path.join(this.options.baseDirectory, `${hash}.json`)
  }

  async _hasCache(cachePath: string) {
    try {
      await fs.access(cachePath, fs.constants.F_OK)
      return true
    } catch {
      return false
    }
  }

  async _getCache(cachePath: string) {
    return await fs.readFile(cachePath, 'utf-8')
  }

  async _setCache(cachePath: string, value: string) {
    return await fs.writeFile(cachePath, value, 'utf-8')
  }
}
